/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.configurator;

import java.util.Properties;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.Connection;
import javax.xml.registry.RegistryService;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.BulkResponse;
import javax.xml.registry.JAXRException;

import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.User;
import javax.xml.registry.infomodel.Service;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sun.wsi.scm.util.JAXRConstants;
import com.sun.wsi.scm.util.WSIConstants;

import com.sun.wsi.scm.configurator.ConfiguratorPortType;

import com.sun.xml.rpc.client.StubBase;
import com.sun.xml.rpc.client.http.HttpClientTransportFactory;

import javax.xml.rpc.Stub;

public class WSIQuery implements JAXRConstants, WSIConstants {
	Log logger = null;
	ConfigOptionsType configOptions = null;

	public static void main(String[] args) {
		new WSIQuery(System.getProperties());
	}

	public WSIQuery(Properties props) {
		LogFactory logFactory = LogFactory.getFactory();
		logger = logFactory.getInstance(LOGGER);

		String mode = props.getProperty("mode");
		if (mode == null) {
			logger.warn("No registry \"mode\" specified; setting to default \"webservice\" mode");
			mode = "webservice";
		}

		if (!(mode.equals("sun") || mode.equals("webservice"))) {
			logger.warn("\"" + mode + "\" mode not supported; setting to default \"webservice\"");
			mode = "webservice";
		}

		logger.info("Starting in \"" + mode + "\" mode");

		try {
			if (mode.equals("sun")) {
				logger.info("Querying for \"Sun Microsystems\" WS-I services ...");
				queryOnSun(props.getProperty("query.manager"), props.getProperty("lifecycle.manager"));
			} else if (mode.equals("webservice")) {
				logger.info("Querying for all WS-I services (web service mode) ...");
				ConfiguratorService_Impl service = new ConfiguratorService_Impl();
				ConfiguratorPortType port = service.getConfiguratorPort();
				String endpoint = props.getProperty("endpoint");
				if (endpoint == null) {
					logger.warn("No configurator endpoint specified. Setting to default configurator endpoint: \"http://localhost:8080/wsi-server/scm/configurator\"");
					endpoint = "http://localhost:8080/wsi-server/scm/configurator";
				}

				((Stub)port)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, endpoint);
				if (logger.isTraceEnabled())
					((StubBase)port)._setTransportFactory(new HttpClientTransportFactory(System.out));
				// default is to get all endpoints from UDDI business registry
				boolean refresh = true;
				if (props.getProperty("refresh") != null)
					refresh = props.getProperty("refresh").equals("true") ?  true : false;

				configOptions = port.getConfigurationOptions(refresh);
			}
		} catch (ConfiguratorFailedFault ex) {
			logger.error(ex);
		} catch (Exception ex) {
			logger.error(ex);
		}

		if (configOptions != null) {
			ConfigOptionType[] configOption = configOptions.getConfigOption();
			logger.info(configOption.length + " services found");
			for (int i=0; i<configOption.length; i++) {
				logger.info("{" + configOption[i].getName() + "}" + configOption[i].getConfigurationEndpoint().get_value());
				if (logger.isDebugEnabled())
					logger.debug("Params: " + configOption[i].getSelectionParms());
				if (configOption[i].getConfigurationEndpoint().getRole() != null)
					if (logger.isDebugEnabled())
						logger.debug("Role: " + configOption[i].getConfigurationEndpoint().getRole().getValue());
					
			}
			logger.info(configOption.length + " services displayed successfully.");
		} else {
			logger.warn("No services found");
		}
	}

	public ConfigOptionsType getConfigurationOptions() { return configOptions; }

	void queryOnSun(String queryManagerProperty, String lifecycleManagerProperty) throws JAXRException {
		Properties props = new Properties();
		props.setProperty(QUERY_MANAGER_URL, queryManagerProperty);
		props.setProperty(LIFECYCLE_MANAGER_URL, lifecycleManagerProperty);

		ConnectionFactory connectionFactory = ConnectionFactory.newInstance();
		connectionFactory.setProperties(props);
		Connection connection = connectionFactory.createConnection();

		RegistryService registryService = connection.getRegistryService();
		BusinessQueryManager queryManager = registryService.getBusinessQueryManager();
		BusinessLifeCycleManager lifecycleManager = registryService.getBusinessLifeCycleManager();

		Collection findQualifiers = new ArrayList();
		findQualifiers.add(FindQualifier.CASE_SENSITIVE_MATCH);
		Collection namePatterns = new ArrayList();
		namePatterns.add("%Sun Microsystems Web Services%");
		BulkResponse response = queryManager.findOrganizations(findQualifiers, namePatterns, null, null, null, null);
		Collection orgs = response.getCollection();

		int orgCount = 0;
		Iterator iter = orgs.iterator();
		while (iter.hasNext()) {
			Organization org = (Organization)iter.next();
			logger.info(++orgCount + "th org: " + org.getName().getValue());
			logger.info("Key: " + org.getKey().getId());

			Collection services = org.getServices();
			Iterator serviceIter = services.iterator();
			int count = 0;
			while (serviceIter.hasNext()) {
				Service service = (Service)serviceIter.next();
				logger.info("\t" + ++count + "th service: " + service.getName().getValue());
			}
		}
	}
}

