/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.manufacturer;

import java.rmi.RemoteException;

import javax.xml.rpc.Stub;
import javax.xml.rpc.JAXRPCException;

import com.sun.xml.rpc.client.StubBase;

import com.sun.wsi.scm.configuration.ConfigurationType;
import com.sun.wsi.scm.configuration.ConfigurationFaultType;

import com.sun.wsi.scm.manufacturer.po.PurchOrdType;

import com.sun.wsi.scm.manufacturer.cb.StartHeaderType;
import com.sun.wsi.scm.manufacturer.cb.CallbackHeaderType;
import com.sun.wsi.scm.manufacturer.cb.CallbackFaultType;

import com.sun.wsi.scm.manufacturer.sn.ShipmentNoticeType;
import com.sun.wsi.scm.manufacturer.sn.ItemList;

import com.sun.wsi.scm.util.WSIConstants;

import com.sun.xml.rpc.client.http.HttpClientTransportFactory;

import java.net.UnknownHostException;

public class CallbackInvoker implements Runnable, WSIConstants {
	ShipmentNoticeType shipmentNotice;
	CallbackHeaderType callbackHeader;

	PurchOrdType purchaseOrder;
	ConfigurationType configurationHeader;
	StartHeaderType startHeader;

	WarehouseCallbackPortType_Stub warehouseCallbackStub;

	public CallbackInvoker(PurchOrdType purchaseOrder, ConfigurationType configurationHeader, StartHeaderType startHeader) {
		this.purchaseOrder = purchaseOrder;
		this.configurationHeader = configurationHeader;
		this.startHeader = startHeader;

		com.sun.wsi.scm.manufacturer.po.ItemList poItemList = purchaseOrder.getItems();
		com.sun.wsi.scm.manufacturer.po.Item[] poItems = poItemList.getItem();

		com.sun.wsi.scm.manufacturer.sn.Item[] snItems = new com.sun.wsi.scm.manufacturer.sn.Item[poItems.length];

		for (int i=0; i<poItems.length; i++) {
			snItems[i] = new com.sun.wsi.scm.manufacturer.sn.Item();
			snItems[i].setID(poItems[i].getID());
			snItems[i].setQty(poItems[i].getQty());
			snItems[i].setPrice(poItems[i].getPrice());
		}
		com.sun.wsi.scm.manufacturer.sn.ItemList snItemList = new com.sun.wsi.scm.manufacturer.sn.ItemList();
		snItemList.setItem(snItems);

		shipmentNotice = new ShipmentNoticeType();
		shipmentNotice.setShipNum("123");
		shipmentNotice.setOrderNum(purchaseOrder.getOrderNum());
		shipmentNotice.setCustomerRef(purchaseOrder.getCustomerRef());
		shipmentNotice.setItems(snItemList);
		shipmentNotice.setTotal(purchaseOrder.getTotal());

		callbackHeader = new CallbackHeaderType();
		callbackHeader.setConversationID(startHeader.getConversationID());

		ManufacturerService_Impl service = new ManufacturerService_Impl();
		warehouseCallbackStub = (WarehouseCallbackPortType_Stub)service.getWarehouseCallbackPort();
		((Stub)warehouseCallbackStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, startHeader.getCallbackLocation());
		//warehouseCallbackStub._setTransportFactory(new HttpClientTransportFactory(System.out));
	}

	public void run() {
		try {
			Thread.currentThread().sleep(CALLBACK_DELAY);
			boolean status = warehouseCallbackStub.submitSN(shipmentNotice, configurationHeader, callbackHeader);
		} catch (InterruptedException ex) {
			ex.printStackTrace(System.out);
		} catch (ConfigurationFaultType ex) {
			ex.printStackTrace(System.out);
		} catch (CallbackFaultType ex) {
			ex.printStackTrace(System.out);
		} catch (RemoteException ex) {
			ex.printStackTrace(System.out);
		} catch (Throwable ex) {
			ex.printStackTrace(System.out);
		}
	}
}


