/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.manufacturer;

import com.sun.wsi.scm.manufacturer.sn.ShipmentNoticeType;

import com.sun.wsi.scm.manufacturer.po.SubmitPOFaultType_Type;

import com.sun.wsi.scm.configuration.ConfigurationType;
import com.sun.wsi.scm.configuration.ConfigurationFaultType;
import com.sun.wsi.scm.configuration.ConfigurationEndpointRole;

import com.sun.wsi.scm.manufacturer.cb.CallbackHeaderType;
import com.sun.wsi.scm.manufacturer.cb.CallbackFaultType;

import javax.xml.rpc.Stub;
import javax.xml.rpc.server.ServiceLifecycle;

import com.sun.xml.rpc.client.StubBase;

import com.sun.wsi.scm.logging.LogEventRequestType;
import com.sun.wsi.scm.logging.LoggingFacilityService_Impl;
import com.sun.wsi.scm.logging.LoggingFacilityLogPortType;

import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Vector;

import com.sun.wsi.scm.util.ConfigurationValidator;
import com.sun.wsi.scm.util.WSIConstants;
import com.sun.wsi.scm.util.DBConnectionPool;

import java.rmi.RemoteException;

import com.sun.xml.rpc.client.http.HttpClientTransportFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class WarehouseCallbackPortTypeImpl implements WarehouseCallbackPortType, ServiceLifecycle, WSIConstants {

	LoggingFacilityLogPortType logStub;
	LogEventRequestType logRequest = new LogEventRequestType();

	Log logger = null;

	DBConnectionPool dbPool = null;

	public void init(Object context) {
		logger = LogFactory.getLog(LOGGER);

		LoggingFacilityService_Impl logService = new LoggingFacilityService_Impl();
		logStub = logService.getLoggingFacilityPort();
		if (logger.isTraceEnabled())
			((StubBase)logStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		dbPool = DBConnectionPool.getInstance(context);
	}

	public void destroy() { }

	public boolean submitSN(ShipmentNoticeType shipmentNotice, ConfigurationType configurationHeader, CallbackHeaderType callbackHeader) throws ConfigurationFaultType, CallbackFaultType, RemoteException {

		if (logger.isTraceEnabled())
			logger.trace("WarehouseCallback.submitSN");
		// ** - Validate the configurationHeader
		Hashtable roles = null;
		roles = ConfigurationValidator.validateHeader(configurationHeader);

		// Set the endpoint address of the logging facility
		// from the configurationHeader
		((Stub)logStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.LoggingFacility));

		Vector setters = new Vector();
		setters.add(callbackHeader.getConversationID());
		ArrayList list = dbPool.query(DBConnectionPool.CALLBACK, setters);
		String warehouse = "WarehouseX";
		String manufacturer = "ManufacturerX";
		if ((list != null) && (list.size() > 0)) {
			warehouse = (String)list.get(0);
			manufacturer = (String)list.get(1);
		}

		logRequest.setDemoUserID(configurationHeader.getUserId());
		logRequest.setServiceID(warehouse + "." + SUBMIT_SN);
		logRequest.setEventID("UC3-7-1");
		logRequest.setEventDescription(warehouse + " has received notice that " + StringConverter.getProductNumbersAsString(shipmentNotice.getItems()) + " has been shipped by " + manufacturer);
		logStub.logEvent(logRequest);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());

		logRequest.setEventID("UC3-7-2");
		logRequest.setEventDescription(warehouse + " has replenished stock for product " + StringConverter.getProductNumbersAsString(shipmentNotice.getItems()));
		logStub.logEvent(logRequest);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());

		return true;
	}

	//public boolean errorPO(SubmitPOFaultType_Type type, ConfigurationType configurationHeader, CallbackHeaderType callbackHeader) throws ConfigurationFaultType, CallbackFaultType {
	public boolean errorPO(SubmitPOFaultType_Type type, CallbackHeaderType callbackHeader) throws CallbackFaultType {
		/*
		// ** - Validate the configurationHeader
		Hashtable roles = null;
		roles = ConfigurationValidator.validateHeader(configurationHeader);

		// Set the endpoint address of the logging facility
		// from the configurationHeader
		((Stub)logStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.LoggingFacility));
		logRequest.setDemoUserID(configurationHeader.getUserId());
		logRequest.setServiceID(WAREHOUSEA_ERROR_PO);
		logRequest.setEventID("UC3-7-4");
		logRequest.setEventDescription("WarehouseA is unable to correlate the notification of a shipping error with a pending replenishment request");
		logStub.logEvent(logRequest);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());
		*/

		return true;
	}
}

