/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.retailer;

import java.util.Vector;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.xml.rpc.Stub;
import com.sun.xml.rpc.client.StubBase;

import java.math.BigDecimal;
import java.math.BigInteger;

import java.rmi.RemoteException;

import javax.xml.rpc.JAXRPCException;
import javax.xml.rpc.server.ServiceLifecycle;
import javax.xml.rpc.soap.SOAPFaultException;

import com.sun.wsi.scm.retailer.RetailerPortType;
import com.sun.wsi.scm.retailer.CatalogType;
import com.sun.wsi.scm.retailer.CatalogItem;
import com.sun.wsi.scm.retailer.PartsOrderType;
import com.sun.wsi.scm.retailer.PartsOrderResponseType;
import com.sun.wsi.scm.retailer.CustomerDetailsType;
import com.sun.wsi.scm.retailer.PartsOrderResponseItem;
import com.sun.wsi.scm.retailer.PartsOrderItem;

import com.sun.wsi.scm.configuration.ConfigurationFaultType;
import com.sun.wsi.scm.configuration.ConfigurationType;
import com.sun.wsi.scm.configuration.ConfigurationEndpointRole;
import com.sun.wsi.scm.configuration.ConfigurationEndpointType;

import com.sun.wsi.scm.logging.LogEventRequestType;
import com.sun.wsi.scm.logging.LoggingFacilityService_Impl;
import com.sun.wsi.scm.logging.LoggingFacilityLogPortType;

import com.sun.wsi.scm.util.ConfigurationValidator;
import com.sun.wsi.scm.util.WSIConstants;

import com.sun.wsi.scm.warehouse.Item;
import com.sun.wsi.scm.warehouse.ItemList;
import com.sun.wsi.scm.warehouse.ItemShippingStatus;
import com.sun.wsi.scm.warehouse.ItemShippingStatusList;
import com.sun.wsi.scm.warehouse.WarehouseService_Impl;
import com.sun.wsi.scm.warehouse.WarehouseShipmentsPortType;

import com.sun.xml.rpc.client.http.HttpClientTransportFactory;

import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.Name;
import javax.xml.soap.Detail;
import javax.xml.soap.DetailEntry;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class RetailerPortTypeImpl implements RetailerPortType, ServiceLifecycle, WSIConstants {
	LoggingFacilityLogPortType logStub = null;
	LogEventRequestType logRequest = new LogEventRequestType();

	Log logger = null;

	public void init(Object context) {
		logger = LogFactory.getLog(LOGGER);

		LoggingFacilityService_Impl logService = new LoggingFacilityService_Impl();
		logStub = logService.getLoggingFacilityPort();

		if (logger.isTraceEnabled())
			((StubBase)logStub)._setTransportFactory(new HttpClientTransportFactory(System.out));
	}

	public void destroy() { }

	public CatalogType getCatalog() {

		if (logger.isTraceEnabled())
			logger.trace(RETAILER_GET_CATALOG);

		CatalogType catalog = new CatalogType();

		CatalogItem[] catalogItems = new CatalogItem[10];
		for (int i=0; i<10; i++) {
			catalogItems[i] = new CatalogItem();
		}

		catalogItems[0].setProductNumber(new BigInteger("605001"));
		catalogItems[0].setName("TV,BrandA");
		catalogItems[0].setBrand("BrandA");
		catalogItems[0].setDescription("24\", color, advanced velocity scan modulated");
		catalogItems[0].setCategory("TV");
		catalogItems[0].setPrice((new BigDecimal(299.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[1].setProductNumber(new BigInteger("605002"));
		catalogItems[1].setName("TV,BrandB");
		catalogItems[1].setBrand("BrandB");
		catalogItems[1].setDescription("32\", super slim flat panel plasma");
		catalogItems[1].setCategory("TV");
		catalogItems[1].setPrice((new BigDecimal(1499.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[2].setProductNumber(new BigInteger("605003"));
		catalogItems[2].setName("TV,BrandC");
		catalogItems[2].setBrand("BrandC");
		catalogItems[2].setDescription("50\", plasma display");
		catalogItems[2].setCategory("TV");
		catalogItems[2].setPrice((new BigDecimal(5725.98)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[3].setProductNumber(new BigInteger("605004"));
		catalogItems[3].setName("VCR,BrandA");
		catalogItems[3].setBrand("BrandA");
		catalogItems[3].setDescription("S-VHS");
		catalogItems[3].setCategory("VCR");
		catalogItems[3].setPrice((new BigDecimal(199.95)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[4].setProductNumber(new BigInteger("605005"));
		catalogItems[4].setName("VCR,BrandB");
		catalogItems[4].setBrand("BrandB");
		catalogItems[4].setDescription("HiFi, S-VHS");
		catalogItems[4].setCategory("VCR");
		catalogItems[4].setPrice((new BigDecimal(400.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[5].setProductNumber(new BigInteger("605006"));
		catalogItems[5].setName("VCR,BrandC");
		catalogItems[5].setBrand("BrandC");
		catalogItems[5].setDescription("S-VHS, MINDY");
		catalogItems[5].setCategory("VCR");
		catalogItems[5].setPrice((new BigDecimal(949.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[6].setProductNumber(new BigInteger("605007"));
		catalogItems[6].setName("DVD,BrandA");
		catalogItems[6].setBrand("BrandA");
		catalogItems[6].setDescription("DVD-Player w/ built-in Dolby Digital decoder");
		catalogItems[6].setCategory("DVD");
		catalogItems[6].setPrice((new BigDecimal(100.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[7].setProductNumber(new BigInteger("605008"));
		catalogItems[7].setName("DVD,BrandB");
		catalogItems[7].setBrand("BrandB");
		catalogItems[7].setDescription("Plays DVD-Video discs, CDs, stereo and music");
		catalogItems[7].setCategory("DVD");
		catalogItems[7].setPrice((new BigDecimal(200.00)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[8].setProductNumber(new BigInteger("605009"));
		catalogItems[8].setName("DVD,BrandC");
		catalogItems[8].setBrand("BrandC");
		catalogItems[8].setDescription("DVD Player with SmoothSlow forward/reverse; Digital Video Enhancer; DVD/CD test; Custom parental control (20-disc); Digital cinema sound modes");
		catalogItems[8].setCategory("DVD");
		catalogItems[8].setPrice((new BigDecimal(1499.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalogItems[9].setProductNumber(new BigInteger("605010"));
		catalogItems[9].setName("TV,Brand4");
		catalogItems[9].setBrand("Brand4");
		catalogItems[9].setDescription("Designated invalid product code that is allowed to appear in the catalog, but is unable to be ordered");
		catalogItems[9].setCategory("TV");
		catalogItems[9].setPrice((new BigDecimal(149.99)).setScale(2, BigDecimal.ROUND_HALF_UP));

		catalog.setItem(catalogItems);

		return catalog;
	}

	private Vector getProductNumbers() {
		Vector v = new Vector();
		v.add(new BigInteger("605001"));
		v.add(new BigInteger("605002"));
		v.add(new BigInteger("605003"));
		v.add(new BigInteger("605004"));
		v.add(new BigInteger("605005"));
		v.add(new BigInteger("605006"));
		v.add(new BigInteger("605007"));
		v.add(new BigInteger("605008"));
		v.add(new BigInteger("605009"));
		// 605010 is an invalid product
		//v.add(new BigInteger("605010"));

		return v;
	}

	public PartsOrderResponseType submitOrder(PartsOrderType partsOrder, CustomerDetailsType customerDetails, ConfigurationType configurationHeader) throws BadOrderFault, InvalidProductCodeType, ConfigurationFaultType, RemoteException {

		if (logger.isTraceEnabled())
			logger.trace("Starting a new submitOrder ...");
		// ** - Validate the configurationHeader
		Hashtable roles = null;
		roles = ConfigurationValidator.validateHeader(configurationHeader);

		// Set the endpoint address of the logging facility
		// from the configurationHeader
		((Stub)logStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, (String)roles.get(ConfigurationEndpointRole.LoggingFacility));
		if (logger.isDebugEnabled()) {
			logger.debug("Logging Facility: " + (String)roles.get(ConfigurationEndpointRole.LoggingFacility));
			logger.debug("Retailer: " + (String)roles.get(ConfigurationEndpointRole.Retailer));
			logger.debug("WarehouseA: " + (String)roles.get(ConfigurationEndpointRole.WarehouseA));
			logger.debug("WarehouseB: " + (String)roles.get(ConfigurationEndpointRole.WarehouseB));
			logger.debug("WarehouseC: " + (String)roles.get(ConfigurationEndpointRole.WarehouseC));
			logger.debug("ManufacturerA: " + (String)roles.get(ConfigurationEndpointRole.ManufacturerA));
			logger.debug("ManufacturerB: " + (String)roles.get(ConfigurationEndpointRole.ManufacturerB));
			logger.debug("ManufacturerC: " + (String)roles.get(ConfigurationEndpointRole.ManufacturerC));
		}

		PartsOrderItem[] partsOrderList = partsOrder.getItem();

		String desc = "Order placed by " + customerDetails.getCustnbr() + " for ";
		for (int i=0; i<partsOrderList.length; i++) {
			desc += partsOrderList[i].getProductNumber();
			desc += (i == (partsOrderList.length-1)) ? "" : ", ";
		}

		logRequest.setDemoUserID(configurationHeader.getUserId());
		logRequest.setServiceID(RETAILER_SUBMIT_ORDER);
		logRequest.setEventID("UC1-5");
		logRequest.setEventDescription(desc);
		logStub.logEvent(logRequest);
		if (logger.isDebugEnabled())
			logger.debug(logRequest.getEventDescription());

		// ** - Validate the order

		// A BadOrder fault is returned if the order
		// contains no line items
		if (partsOrderList == null)
			throw new BadOrderFault("No line items in the order");


		// Throw InvalidProductCode fault, if it contains a line item
		// with an invalid (i.e. unknown) product code
		Vector productNumbersVector = getProductNumbers();
		for (int i=0; i<partsOrderList.length; i++) {
			if (!productNumbersVector.contains(partsOrderList[i].getProductNumber())) {
				logRequest.setEventID("UC1-ALT1-1");
				logRequest.setEventDescription("Order placed by " + customerDetails.getCustnbr() + " is rejected as " + partsOrderList[i].getProductNumber() + " is not a valid product and processing has terminated");
				logStub.logEvent(logRequest);
				if (logger.isDebugEnabled())
					logger.debug(logRequest.getEventDescription());

				// Throw SOAPFaultException as env:Client fault code
				// is required on the client side
				try {
					SOAPFactory soapFactory = SOAPFactory.newInstance();
					Detail detail = soapFactory.createDetail();
					QName faultcode = new QName(SOAP_ENVELOPE_NAMESPACE, "Client");
					Name name = soapFactory.createName("InvalidProductCode", "ns1", RETAIL_ORDER_NAMESPACE);
					DetailEntry detailEntry = detail.addDetailEntry(name);
					//detailEntry.addNamespaceDeclaration("ns1", RETAIL_ORDER_NAMESPACE);
					SOAPElement soapElement = detailEntry.addChildElement("Reason", "ns1", RETAIL_ORDER_NAMESPACE);
					soapElement.addTextNode("InvalidProductCode");
					soapElement = detailEntry.addChildElement("ProductNumber", "ns1", RETAIL_ORDER_NAMESPACE);
					soapElement.addTextNode(partsOrderList[i].getProductNumber().toString());

					throw new SOAPFaultException(faultcode, "com.sun.wsi.scm.retailer.InvalidProductCodeType", RETAILER_SUBMIT_ORDER, detail);
				} catch (SOAPException ex) {
					logger.error(ex.getMessage(), ex);
				}
				//throw new InvalidProductCodeType(new Reason_Type("InvalidProductCode"), partsOrderList[i].getProductNumber());
			}
		}

		// Create the list of items requested
		if (logger.isTraceEnabled())
			logger.trace("Creating the list of items requested");
		Item[] requestItemArray = new Item[partsOrderList.length];
		for (int i=0; i<partsOrderList.length; i++) {
			requestItemArray[i] = new Item();
			requestItemArray[i].setProductNumber(partsOrderList[i].getProductNumber());
			requestItemArray[i].setQuantity(partsOrderList[i].getQuantity().intValue());
			// partsOrderList[i].getPrice() is not getting used anywhere
			// and that's ok since the two prices may differ
		}
		ItemList requestItemList = new ItemList();
		requestItemList.setItem(requestItemArray);

		// ** - Initialize the response
		// Used to correlate with the list of items that
		// could not be ordered by a particular warehouse
		Hashtable requestHash = new Hashtable();
		Hashtable responseHash = new Hashtable();

		for (int i=0; i<partsOrderList.length; i++) {
			// Initialize the requestHash with the original request
			requestHash.put(partsOrderList[i].getProductNumber(), partsOrderList[i]);

			// Initialize the response with productNumber
			PartsOrderResponseItem responseItem = new PartsOrderResponseItem();
			responseItem.setProductNumber(partsOrderList[i].getProductNumber());

			// Initialize the responseHash with the initialized response
			responseHash.put(partsOrderList[i].getProductNumber(), responseItem);
		}

		WarehouseService_Impl warehouseService = new WarehouseService_Impl();

		// configure WarehouseA stub
		WarehouseShipmentsPortType warehouseAStub = warehouseService.getWarehouseAPort();
		((Stub)warehouseAStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, roles.get(ConfigurationEndpointRole.WarehouseA));
		if (logger.isTraceEnabled())
			((StubBase)warehouseAStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		// configure WarehouseB stub
		WarehouseShipmentsPortType warehouseBStub = warehouseService.getWarehouseBPort();
		((Stub)warehouseBStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, roles.get(ConfigurationEndpointRole.WarehouseB));
		if (logger.isTraceEnabled())
			((StubBase)warehouseBStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		// configure WarehouseC stub
		WarehouseShipmentsPortType warehouseCStub = warehouseService.getWarehouseCPort();
		((Stub)warehouseCStub)._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, roles.get(ConfigurationEndpointRole.WarehouseC));
		if (logger.isTraceEnabled())
			((StubBase)warehouseCStub)._setTransportFactory(new HttpClientTransportFactory(System.out));

		// Process order with WarehouseA
		ItemList unshippedItemList = processOrder(warehouseAStub, requestItemList, customerDetails.getCustnbr(), configurationHeader, requestHash, responseHash, ConfigurationEndpointRole._WarehouseAString);

		// If WarehouseA is not able to ship all the goods,
		// then WarehouseB is invoked
		if (unshippedItemList.getItem().length > 0) {
			unshippedItemList = processOrder(warehouseBStub, unshippedItemList, customerDetails.getCustnbr(), configurationHeader, requestHash, responseHash, ConfigurationEndpointRole._WarehouseBString);

			// If WarehouseB is not able to ship the remaining goods,
			// then WarehouseC is invoked
			if (unshippedItemList.getItem().length > 0) {
				unshippedItemList = processOrder(warehouseCStub, unshippedItemList, customerDetails.getCustnbr(), configurationHeader, requestHash, responseHash, ConfigurationEndpointRole._WarehouseCString);

			}
		}

		// If there are any goods remaining to be shipped, they are 
		// marked with 0 quantity and "Insufficient stock" comment
		if (unshippedItemList.getItem().length > 0) {
			// If none of the items could be shipped by any of the warehouses
			// then logEvent is invoked
			if (partsOrder.getItem().length == unshippedItemList.getItem().length) {
				logRequest.setEventID("UC1-ALT2-1");
				logRequest.setEventDescription("There is no availability for any products in the order placed by " + customerDetails.getCustnbr() + " and processing is terminated");
				logStub.logEvent(logRequest);
				if (logger.isDebugEnabled())
					logger.debug(logRequest.getEventDescription());
			} else {
				logRequest.setEventID("UC1-9");
				logRequest.setEventDescription("Processing of the order from " + customerDetails.getCustnbr() + " has finished normally.");
				logStub.logEvent(logRequest);
				if (logger.isDebugEnabled())
					logger.debug(logRequest.getEventDescription());
			}
			completeOrder(unshippedItemList, requestHash, responseHash);
		} else {		// all the items are shipped correctly
			logRequest.setEventID("UC1-9");
			logRequest.setEventDescription("Processing of the order from " + customerDetails.getCustnbr() + " has finished normally.");
			logStub.logEvent(logRequest);
			if (logger.isDebugEnabled())
				logger.debug(logRequest.getEventDescription());
		}

		// Convert the responseHash to PartsOrderResponseItem[] and return
		PartsOrderResponseItem[] partsOrderResponse = new PartsOrderResponseItem[responseHash.size()];

		int count = 0;
		Enumeration responseEnum = responseHash.elements();
		while (responseEnum.hasMoreElements()) {
			partsOrderResponse[count] = (PartsOrderResponseItem)responseEnum.nextElement();
			double price = partsOrderResponse[count].getQuantity().floatValue() * partsOrderResponse[count].getPrice().floatValue();

			partsOrderResponse[count++].setPrice((new BigDecimal(price)).setScale(2, BigDecimal.ROUND_HALF_UP));
		}


		PartsOrderResponseType response = new PartsOrderResponseType();
		response.setItem(partsOrderResponse);

		return response;
	}

	String getProductNumbersAsString(ItemList itemList) {
		String desc = "";

		Item[] items = itemList.getItem();
		for (int i=0; i<items.length; i++) {
			desc += items[i].getProductNumber();
			if (i != (items.length-1))
				desc += ", ";
		}

		return desc;
	}

	/**
	* 1). Invoke warehouse stub to ship the goods
	* 2). Items shipped by a warehouse are marked as success
	*     in responseHash
	* 3). Items not not shipped are collected in unshippedItems
	* 4). Items not shipped by a warehouse are then populated
	*     with the original quantity and returned back
	*/
	private ItemList processOrder(WarehouseShipmentsPortType warehouseStub, ItemList requestItemList, String customerNumber, ConfigurationType configurationHeader, Hashtable requestHash, Hashtable responseHash, String role) throws ConfigurationFaultType, RemoteException {

		if (logger.isTraceEnabled())
			logger.trace("Processing order for " + role);

		// Invoke the warehouse stub
		ItemShippingStatusList itemShippingStatusList = null;

		// Cast the warehouse to the appropriate stub based upon the role
		if (role.equals(ConfigurationEndpointRole._WarehouseAString)) {
			itemShippingStatusList = warehouseStub.shipGoods(requestItemList, customerNumber, configurationHeader);
		} else if (role.equals(ConfigurationEndpointRole._WarehouseBString)) {
			itemShippingStatusList = warehouseStub.shipGoods(requestItemList, customerNumber, configurationHeader);
		} else if (role.equals(ConfigurationEndpointRole._WarehouseCString)) {
			itemShippingStatusList = warehouseStub.shipGoods(requestItemList, customerNumber, configurationHeader);
		} else {
			logger.error("Invalid role");
		}
			

		ItemShippingStatus[] itemShippingStatusArray = itemShippingStatusList.getItemStatus();

		ArrayList unshippedItems = new ArrayList();
		// Sets a valid value in the responseHash for all the items 
		// shipped by the warehouse defined by "role" parameter
		for (int i=0; i<itemShippingStatusArray.length; i++) {
			BigInteger productNumber = itemShippingStatusArray[i].getProductNumber();
			if (itemShippingStatusArray[i].isStatus()) {
				PartsOrderItem requestItem = (PartsOrderItem)requestHash.get(productNumber);
				PartsOrderResponseItem responseItem = (PartsOrderResponseItem)responseHash.get(productNumber);
				responseItem.setQuantity(requestItem.getQuantity());
				responseItem.setPrice(requestItem.getPrice());
				responseItem.setComment("In stock from " + role);

				responseHash.put(productNumber, responseItem);
			} else {
				unshippedItems.add(itemShippingStatusArray[i].getProductNumber());
			}
		}

		// Collect products not shipped by a warehouse. Quanity is
		// obtained from the original request as ItemShippingStatus
		// (response from Warehouse) does not contain quantity
		Item[] requestItemArray = requestItemList.getItem();
		Hashtable quantityHash = new Hashtable();
		for (int i=0; i<requestItemArray.length; i++)
			quantityHash.put(requestItemArray[i].getProductNumber(), new Integer(requestItemArray[i].getQuantity()));

		Item[] unshippedItemArray = new Item[unshippedItems.size()];
		for (int i=0; i<unshippedItems.size(); i++) {
			BigInteger productNumber = (BigInteger)unshippedItems.get(i);
			unshippedItemArray[i] = new Item();
			unshippedItemArray[i].setProductNumber(productNumber);

			// what about the quantity
			Integer quantityInteger = (Integer)quantityHash.get(new Integer(productNumber.intValue()));
			unshippedItemArray[i].setQuantity(((Integer)quantityHash.get(productNumber)).intValue());
		}

		ItemList unshippedItemList = new ItemList();
		unshippedItemList.setItem(unshippedItemArray);

		return unshippedItemList;
	}

	/**
	* For items not shipped by any of the warehouses, 0 quantity 
	* and "Insufficient stock" comment is specified in the response
	*/
	private void completeOrder(ItemList unshippedItemList, Hashtable requestHash, Hashtable responseHash) {
		if (logger.isTraceEnabled())
			logger.trace("Completing the order processing");

		Item[] unshippedItemArray = unshippedItemList.getItem();

		for (int i=0; i<unshippedItemArray.length; i++) {
			BigInteger productNumber = unshippedItemArray[i].getProductNumber();
			PartsOrderItem requestItem = (PartsOrderItem)requestHash.get(productNumber);
			PartsOrderResponseItem responseItem = (PartsOrderResponseItem)responseHash.get(productNumber);
			responseItem.setQuantity(BigInteger.ZERO);
			responseItem.setPrice(requestItem.getPrice());
			responseItem.setComment("Insufficient stock");

			responseHash.put(productNumber, responseItem);
		}
	}
}

